/**
 * A class that defines integers as implementing IOrderable.
 * 
 * @author Gloss
 * @version 1.1 - 11/Jul/2013 asantos
 **/
public class MyInteger implements IOrderable<MyInteger>, Cloneable {
	
	/**
	 * The internal representation of MyInteger.
	 **/
	private int num;
	
	/**
	 * The constructor.
	 * @param i - the number to be represented
	 **/
	public MyInteger(int i) {
		num = i;
	}
	
	/**
	 * @param k - the number to be compared with.
	 * 
	 * @return true if current number is greater or equal to k, false otherwise 
	 **/
	public boolean isGreaterEq(MyInteger k) {
		return num >= k.num;
	}
	
	/**
	 * Creates a copy of MyInteger.
	 * 
	 * @return the clone of MyInteger.
	 **/
	@Override
	public Object clone() {
		try {
			return super.clone();
		} catch (CloneNotSupportedException e) {
			throw new InternalError(e.getMessage());
		}
	}
	
	/**
     * Hashcode for MyInteger
     * 
     * @return hashcode for the MyInteger
     */
	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + num;
		return result;
	}
	
	/**
	 * Compares if the object is equal to MyInteger.
	 * 
	 * @param obj - the object to be compared
	 * 
	 * @return true if they are equal, otherwise return false.
	 **/
	@Override
	public boolean equals(Object obj) {
		return this == obj || ((obj instanceof MyInteger) && num == ((MyInteger) obj).num);
	}
	
	/**
	 * The textual representation of MyInteger.
	 * 
	 * @return String representing MyInteger.
	 **/
	@Override
	public String toString() {
		return Integer.toString(num);
	}
}
