/**
 * Class representing a Planet with a name and diameter
 * 
 * @author Gloss
 * @version 1.0 - 11/Jul/2013
 */
public class Planet implements IOrderable<Planet>, Cloneable {
	
	/**
	 * Diameter of the planet
	 */
	private int diameter;
	
	/**
	 * Name of the planet
	 */
	private String name;
	
	/**
	 * Constructor.
	 * 
	 * @param diameter - diameter of the Planet.
	 * @param name - name of the Planet.
	 */
	public Planet(int diameter, String name) {
		this.diameter = diameter;
		this.name = name;
	}

	/**
	 * Returns the diameter of the Planet.
	 * 
	 * @return diameter of the Planet.
	 */
	public int getDiameter() {
		return diameter;
	}

	/**
	 * Returns the name of the Planet.
	 * 
	 * @return name of the Planet.
	 */
	public String getName() {
		return name;
	}
	
	/**
	 * Checks if this Planet's diameter is larger than the other. 
	 * In case of the same diameter, uses the name for comparison.
	 * 
	 * @param other - The other Planet to compare to.
	 * @return <tt>True</tt> in case this Planet is larger or has the same diameter 
	 * 						and a "larger" name according to String comparison, or
	 * 						if both name and diameter are equal
	 * 			<tt>False</tt> otherwise.
	 */
	public boolean isGreaterEq(Planet other) {

		if (equals(other))
			return true;
		if (this.diameter == other.diameter)
			return (this.name.compareTo(other.name) >= 0);
		else
			return (this.diameter > other.diameter);
	}

	/**
	 * Compares this Planet to Object to see if they are equal
	 * @param other - Object to compare to this Planet
	 * @return true if other is equal to this Planet, false otherwise
	 */
	@Override
	public boolean equals(Object other) {
		return this == other || (other != null && (other instanceof Planet)
				&& this.diameter == ((Planet) other).getDiameter()
				&& this.name.equals(((Planet) other).getName()) );
	}
	
	/**
	 * Calculates the hash code of this Planet
	 * @return hash code for this Planet
	 */
	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + diameter;
		result = prime * result + ((name == null) ? 0 : name.hashCode());
		return result;
	}

	/**
	 * Makes a copy of this Planet
	 * @return clone of this Planet
	 */
	@Override
	public Planet clone() {
		try {
			return (Planet) super.clone();
		} catch (CloneNotSupportedException e) {
			throw new InternalError(e.getMessage());
		}
	}

	/**
	 * Textual representation of Planet
	 * @return String representing the Planet
	 */
	@Override
	public String toString() {
		return "Planet " + name + ": " + diameter;
	}
}