import java.util.Random;

/**
 * A class that implements the random test for TreeSet class
 * using planets and myintegers as parameters
 * 
 * @author Gloss
 * @version 1.1 11/Jul/13 asantos
 **/
public class TreeSetRandomTest {

	/**
	 * How many tests do we want to perform.
	 */
	private int howMany;

	/**
	 * Max operations.
	 */
	private static int MAX_OPERATIONS = 6;

	/**
	 * Our random object to generate random int.
	 */
	private Random rand;


	/**
	 * TreeSet object with myintegers.
	 */
	private TreeSet<MyInteger> treeI;

	/**
	 * Max integer to insert in the set.
	 */
	private static final int MAX_INT = 15;


	/**
	 * TreeSet object with planets.
	 */
	private TreeSet<Planet> treeP;

	/**
	 * Name and dimension pool for using in the generation of planets.
	 */
	private static final String[] namePool = { "X", "Y", "Z", "W",
		"XX", "YY", "ZZ" };
	private static final int[] dimPool = { 1, 20, 300, 4000};


	/**
	 * Maximum number of elements in a random generated tree
	 */
	private static final int MAX_ELS = 6;


	/**
	 * Constructor.
	 * 
	 * @param treeSet - the treebag to be tested.
	 * @param howMany - how many tests we want to perform.
	 */
	public TreeSetRandomTest(int howMany) {
		this.rand = new Random();
		this.treeI = newTreeSetInteger();
		this.treeP = newTreeSetPlanet();
		this.howMany = howMany;
	}

	/**
	 * Runs the tests.
	 */
	public void run() {

		for (int i = 0; i < howMany; i++) {

			//choose the tree to use in the iteration
			boolean isIntTree = rand.nextBoolean();

			switch (rand.nextInt(MAX_OPERATIONS)) {

			// show the tree.
			case 0: {
				System.out.println("Printing the set");
				TreeSet<?> tree = isIntTree?treeI:treeP;
				System.out.println(tree);
				break;
			}
			// insert in the tree.
			case 1: {
				if (isIntTree){
					MyInteger el = newMyInteger();
					treeI.insert(el);
					System.out.println("Inserted " + el + " in the integer set");
				} else {
					Planet el = newPlanet();
					treeP.insert(el);
					System.out.println("Inserted " + el + " in the planet set");
				}
				break;
			}
			// is the  set empty.
			case 2: {
				TreeSet<?> tree = isIntTree?treeI:treeP;
				System.out.println("Set is " + 
						(tree.isEmpty()? "" : "not ") + "empty ");
				break;
			}
			// is element in the tree.
			case 3: {
				if (isIntTree){
					MyInteger el = newMyInteger();
					System.out.println(el + " is " + 
							(treeI.isIn(el)? "" : "not ") + "in the set ");
				} else {
					Planet el = newPlanet();
					System.out.println(el + " is " + 
							(treeP.isIn(el)? "" : "not ") + "in the set ");
				}
				break;
			}
			// get the largest element in the tree.
			case 4: {
				TreeSet<?> tree = isIntTree?treeI:treeP;
				if(!tree.isEmpty())
					System.out.println("The largest element in the set is "
							+ tree.largest());
				break;
			}
			// a new tree
			case 5: {
				if (isIntTree){				
					treeI = newTreeSetInteger();
				} else{				
					treeP = newTreeSetPlanet();
				}
				System.out.println("Start with a new set ");
				break;
			}

			}
			System.out.println();	
		}
	}
	/**
	 * Creates a new treeSet of MyIntegers.
	 * 
	 * @return a new TreeSet
	 **/
	private TreeSet<MyInteger> newTreeSetInteger() {

		TreeSet<MyInteger> resultTree = new TreeSet<MyInteger>();
		int elements = rand.nextInt(MAX_ELS);
		for (int i = 0; i < elements ; i++) {
			resultTree.insert(newMyInteger());
		}
		return resultTree;
	}

	/**
	 * Creates a new treeSet of Planets.
	 * 
	 * @return a new TreeSet
	 **/
	private TreeSet<Planet> newTreeSetPlanet() {

		TreeSet<Planet> resultTree = new TreeSet<Planet>();
		int elements = rand.nextInt(MAX_ELS);
		for (int i = 0; i < elements ; i++) {
			resultTree.insert(newPlanet());
		}
		return resultTree;
	}

	/**
	 * Creates a new random MyInteger.
	 * 
	 * @return a new MyInteger
	 **/
	private MyInteger newMyInteger() {
		return new MyInteger(rand.nextInt(MAX_INT));
	}

	/**
	 * Creates a new Planet.
	 * 
	 * @return a new Planet.
	 */
	private Planet newPlanet() {
		return new Planet(dimPool[rand .nextInt(dimPool.length)], 
				namePool[rand.nextInt(namePool.length)]);
	}
}